(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    //
    //
    //
    class Stack {
        constructor(items) {
            if (items === undefined) {
                this.items = [];
            }
            else if (Stack.isStack(items)) {
                this.items = items.items.slice();
            }
            else if (Array.isArray(items)) {
                this.items = items.slice();
            }
            else {
                this.items = [...items];
            }
        }
        static isStack(o) {
            return o instanceof Stack;
        }
        get count() {
            return this.items.length;
        }
        [Symbol.iterator]() {
            let index = 0;
            return {
                next: () => {
                    return {
                        done: index >= this.count,
                        value: this.items[index++]
                    };
                }
            };
        }
        get(index) {
            if (index < 0 || index >= this.count) {
                throw new Error('The index is out of range.');
            }
            return this.items[index];
        }
        push(item) {
            this.items.push(item);
        }
        pop() {
            if (this.count === 0) {
                throw new Error('The stack is empty.');
            }
            return this.items.pop();
        }
        peek() {
            if (this.count === 0) {
                throw new Error('The stack is empty.');
            }
            return this.items[this.count - 1];
        }
        clear() {
            this.items.splice(0, this.count);
        }
        clone() {
            return new Stack(this);
        }
        toArray() {
            return this.items.slice();
        }
        toString() {
            return this.items.toString();
        }
    }
    exports.Stack = Stack;
});
